#!/usr/bin/env python2.2

#    This file is part of Anarcryst.
#
#    Anarcryst is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    Anarcryst is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Anarcryst; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


from parser import *
import sys, os, string

# helper functions

def createFile(parserobj, num):
    """Create an Anarcryst output file.

    Takes a parser object and the number of the file being generated,
    as arguments; returns a reference to the file descriptor of the file
    thus created."""
    keywords = parserobj.getKw()
    rfn = keywords["rootfilename"].rstrip().lstrip()
    fn = string.join([rfn.rstrip(), str(num), '.dat'], "")
    fd = open(fn, "w") # This is destructive. Deal.
    # dump the header
    for line in parserobj.getHeader():
        fd.write(line)
    return fn, fd # need to pass the filename to the logger

def writeCell(parserobj, fd, logfd, cellx, celly, cellz, maxx, maxy, maxz):
    """ Write a given unit cell to an Anarcryst output file. """

    # now the bulk structure
    bulkgroup = parserobj.getBulk()
    bulkgroup.writeGroup(fd, cellx, celly, cellz, maxx, maxy, maxz)
    # now the varying structure:
    # select a random group for this cell
    disgroups = parserobj.getDisGrp()
    print disgroups
    # log it
    for group in disgroups:
        print group.getVector()
        logfd.write("Cell %s %s %s; Vector %s\n" % (cellx, celly, cellz,
                                                    group.getVector()))
        # and output
        group.writeGroup(fd, cellx, celly, cellz, maxx, maxy, maxz)
        # and now yield control to the calling scope; our work here is done.

def usage():
    sys.stderr.write("""
    Anarcryst -  A disordered-crystal-system generator.

    Author: Andrew Walkingshaw, 2001.
    
    Usage:

    anarcryst inputfile

    See 'format.txt' in your anarcryst distribution for an explanation of the
    Anarcryst input format.

    This program is free software: see license.txt in your distribution for
    details.

""")
    sys.exit(0)

# end 
    
def main():
    # Parse the input file
    try:
        pars = Parser(sys.argv[1])
    except IndexError:
        usage()
        sys.exit(-1)
    pars.parse()

    # Global state variables
    iteration = 1

    # Fill the global keyword-initialised state
    keywords = pars.getKw()
    rootname = keywords["rootfilename"]
    logname = keywords["logfilename"].lstrip().rstrip()
    maxiter = keywords["numoutputs"]
    maxx, maxy, maxz = keywords["numx"], keywords["numy"], keywords["numz"]
    
    # Open the logfile
    logfile = open(logname, "w") # !destructive!
    logfile.write("""
Anarcryst Log

Root Filename: %s
Number of Structures to Generate: %s

""" % (rootname, maxiter))

    while iteration <= maxiter:
        a, b, c = 0, 0, 0 # lattice vectors
        filename, fdesc = createFile(pars, iteration)
        logfile.write("""
Iteration %s started.
Filename: %s

""" % (iteration, filename))
        # loop over the extended cell
        while c < maxz:
            b = 0 # reset, carry to c
            while b < maxy:
                a = 0 # reset, carry to b
                while a < maxx: # iterate over all: nested loops 'r us
                    writeCell(pars, fdesc, logfile, a, b, c, maxx, maxy, maxz)
                    # !!LOG!! - done in writeCell()
                    a += 1
                b+=1
            c+=1
        for line in pars.getFooter():
            fdesc.write(line)
        fdesc.close() # close this output file
        sys.stderr.write("Completed iteration %s\n" % iteration)
        iteration += 1 # back to start; reset lattice vectors, get new file
    # and exit
    sys.stderr.write("Completed all iterations!\n")
    sys.exit(0) # and return. Success!

# So let's see if this works, then
if __name__ == '__main__':
    main()
